# coding: utf8
# 
# This file is released under the terms of the GNU General Public License
# as they appear in http://www.gnu.org/licenses/gpl.txt.
#
# Copyright 2009-2017 erwin.rieger@ibrieger.de
#

# CVS version: $Revision: 1.18 $ $Date: 2017-05-19 16:33:20 $

#
# Messung von sehr kleinen zeitintervallen:
#
# Unter linux bietet time.time() die notwendige aufloesung (kleiner 10uS).
# Unter windows ist time.clock() die bessere wahl (aufloesung kleiner 20uS,
# getestet mit win2000 und XP).
#

import os, sys
import parallel, time


runningOnWindows = False
try:
    os.uname()
except AttributeError:
    # Windows
    runningOnWindows = True
    import winsound


def sound():
    
    if runningOnWindows:
        try:
            winsound.PlaySound("prompt.wav", winsound.SND_FILENAME + winsound.SND_PURGE + winsound.SND_PURGE + winsound.SND_PURGE)
        except RuntimeError, why:
            print "sound(): can't play sound:", why
    else:
        os.system("aplay prompt.wav > /tmp/lichtschranke_aplay.log 2>&1 &")

class PPStopUhr(parallel.Parallel):

    def __init__(self):

        parallel.Parallel.__init__(self)

        """
        self.device = "/dev/parport0"
        try:
            self._fd = os.open(self.device, os.O_RDWR)
        except OSError:
            print "Can't open /dev/parport0, missing ppdev module?"
            sys.exit(0)

        # self.PPEXCL()
        try:
            self.PPCLAIM()
        except IOError:
            print "Can't claim /dev/parport0, missing ppdev module?"
            sys.exit(0)

        self.setDataDir(1)
        self.PPWCONTROL(0)
        """

        self.setData(0)

        #
        # Zeitmessung mit time.time() oder time.clock(), je nach
        # betriebssystem:
        #
        if runningOnWindows:
            self.timeFunc = time.clock
        else:
            self.timeFunc = time.time

    def scanByte(self, t0):

        halfBitCount = 0
        ret = 0

        bit = 8

        pruefSumme = 0 # Anzahl gesetzter bits

        while bit > 0:

            halfBitCount += 1

            tAbtast = t0 + halfBitCount*.00025

            synced = 0

            l1 = self.getInBusy()
            t = self.timeFunc()

            while t < tAbtast:
                t = self.timeFunc()
                l = self.getInBusy()
                if not synced and l1 != l:
                    halfBitCount = 0
                    t0 = t + 0.000125
                    synced = 1

            halfBitCount += 1

            tAbtast = t0 + halfBitCount*.00025

            synced = 0

            l2 = self.getInBusy()
            t = self.timeFunc()

            if l1 == l2:
                # print "scanByte(): ERROR, l1 == l2"
                return None

            while t < tAbtast:
                t = self.timeFunc()
                l = self.getInBusy()
                if not synced and l2 != l:
                    halfBitCount = 0
                    t0 = t + 0.000125
                    synced = 1

            if l2:
                # Got ONE-Bit
                ret += 1 << (bit-1)
                pruefSumme += 1

            bit -= 1

        # print "scanByte(): read byte: ", ret

        return (ret, pruefSumme)

    # Auf startbit warten, bytes mit scanByte() einlesen.
    def recvStartStop(self):

        i = 250
        prevLevel = 0
        tFlanke = startTime = self.timeFunc()
        firstpacket = None
        nSync = 0

        while i > 0 or firstpacket:

            t = self.timeFunc()
            b = self.getInBusy()

            if (t - startTime) > 2.5: # xxx anpassung auf x*tdatagramm
                print "Timeout waiting for datagram in recvStartStop()"
                return None

            if b != prevLevel:
                # print "flanke: ", prevLevel, " -> ", b

                tLevel = t - tFlanke

                prevLevel = b
                tFlanke = t

                # print "tLevel: ", tLevel

                # 2000Hz: tlevel = 0.25ms 
                if tLevel > 0.0002 and tLevel < 0.0003:
                    # Sync sequence
                    # print "TSync:", tLevel
                    i = 250
                    nSync += 1
                    continue

                # Startbit: highpegel fuer 0.75 ms, dann 0.25ms low
                if tLevel > 0.0007 and tLevel < 0.0008 and (not b) and nSync > 1:

                    # print "TStartbit:", int(tLevel * 1000000), "uS", nSync

                    # Wir sind nun in der 2. haelfte des startbits, wir koennen ab jetzt nicht
                    # mehr nach signalflanken suchen, jetzt muss in festen zeitabstaenden abgetastet
                    # werden

                    i = 250
                    nSync = 0

                    t = self.timeFunc()
                    twait = 0.000125 - (tLevel - 0.001)
                    while t - tFlanke < twait:
                        t = self.timeFunc()

                    # Read packetCounter, pruefSumme
                    pCounter = self.scanByte(t)
                    if not pCounter:
                        continue

                    # Read moduleId, pruefSumme
                    moduleId = self.scanByte(self.timeFunc())
                    if not moduleId:
                        continue

                    pruefSumme = moduleId[0] & 0xf
                    moduleId = moduleId[0] >> 4

                    if pruefSumme != pCounter[1]:
                        print "recvStartStop(): ERROR, PruefSumme", pCounter[1], "!=", pruefSumme
                        continue

                    if pCounter[0] > 0 and pCounter[0] <= 200:
                        if firstpacket:
                            print "p2: ", pCounter[0], moduleId, t
                            if firstpacket[0] > pCounter[0] and firstpacket[1] == moduleId:

                                # Korrektur t: t = t - ((200 - pcount) * 12.5 ms)
                                tdiff = ((200 - firstpacket[0]) * 0.0125)

                                print "START / STOP, Korrektur [s]: ", tdiff
                                print ""
                                return (pCounter[0], firstpacket[1], firstpacket[2] - tdiff)
                            else:
                                print "p2 wird p1: ", pCounter[0], moduleId, t
                                firstpacket = (pCounter[0], moduleId, t)
                        else:
                            print "p1: ", pCounter[0], moduleId, t
                            firstpacket = (pCounter[0], moduleId, t)

                    continue

                # Kein Sync und kein Startbit -> Out of Sync 
                # if nSync:
                    # print "Out of Sync at", nSync, int(tLevel * 1000000), "uS"
                nSync = 0

            i -= 1

        # print "recvStartStop() returns after", t - startTime
        return None


if __name__ == "__main__":

    pps = PPStopUhr()

    while True:
        p = pps.recvStartStop()
        if p:
            sound()
            print "MAIN: received start/stop, waiting for 2.5 seconds ...", p
            time.sleep(2.5)
            print "done"
        time.sleep(.1)









