# coding: utf8
# 
# This file is released under the terms of the GNU General Public License
# as they appear in http://www.gnu.org/licenses/gpl.txt.
#
# Copyright 2009-2017 erwin.rieger@ibrieger.de
#

# CVS version: $Revision: 1.27 $ $Date: 2017-05-21 21:18:53 $

#
# Für einen ansehnlichen font muss TK mit truetype unterstützung
# konfiguriert sein.
#

import sys, time, random

def debugging():
    for arg in sys.argv:
        if arg == "debug":
            return True
    return False

from Tkinter import *
import tkMessageBox

if debugging():
    import libstoppuhrdebug as libstoppuhr
else:
    import libstoppuhr

##############################file Meter.py####################
import Tkinter

class Meter(Tkinter.Frame):
    '''A simple progress bar widget.'''
    def __init__(self, master, fillcolor='orchid1', text='', value=0.0, **kw):
        Tkinter.Frame.__init__(self, master, bg='white', width=350, height=20)
        self.configure(**kw)

        self._c = Tkinter.Canvas(self, bg=self['bg'], width=self['width'], height=self['height'],\
            highlightthickness=0, relief='flat', bd=0)
        self._c.pack(fill='x', expand=1)
        self._r = self._c.create_rectangle(1, 1, 1, int(self['height'])-1, fill=fillcolor, width=0)
        self._t = self._c.create_text(int(self['width'])/2, int(self['height'])/2, text='')
        self._c.create_rectangle(0, 0, self['width']-1, int(self['height']-1), outline="black")

        self.set(value, text)

    def set(self, value=0.0, text=None):
        #make the value failsafe:
        if value < 0.0:
            value = 0.0
        elif value > 1.0:
            value = 1.0
        if text == None:
            #if no text is specified get the default percentage string:
            text = str(int(100 * value)) + ' %'
        self._c.coords(self._r, 1, 1, (int(self['width'])-1) * value, int(self['height'])-1)
        self._c.itemconfigure(self._t, text=text)

#####################################################################


class TkStoppUhr:

    def __init__(self, root = None):

        if root == None:
            self.root = Tk()
        else:
            self.root = root

        self.root.title("Zeitmessung - Ing. Buero Rieger")

        # Bestzeit
        group = LabelFrame(self.root, text="Bestzeit", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="both", expand=1)

        self.lbest = Label(group, font=('arial', 100, 'bold'), bg='lightgreen')
        self.lbest.pack(fill=BOTH, expand=1, side="left")


        b = Button(group, text = "Reset", width=10, command = self.reset_best)
        b.pack(padx=10, pady=10) #fill=BOTH, expand=1)

        # Aktuelle Zeit
        group = LabelFrame(self.root, text="Zeit", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="both", expand=1)

        self.lakt = Label(group, font=('arial', 100, 'bold'), bg='lightgreen')
        self.lakt.pack(fill=BOTH, expand=1, side="left") # fill=BOTH, expand=1)

        b = Button(group, text = "Stop", width=10, command = self.stop_watch)
        b.pack(padx=10, pady=10) #fill=BOTH, expand=1)

        b = Button(group, text = "Freigabe", width=10, command = self.freigabe)
        b.pack(padx=10, pady=10) #fill=BOTH, expand=1)

        if debugging():
            b = Button(group, text = "Start/Stop", width=10, command = self.startstop)
            b.pack(padx=10, pady=10) #fill=BOTH, expand=1)

        self.lbest.config(text="00:00:00")
        self.lakt.config(text="00:00:00")

        # Geschwindigkeit in m/s
        group = LabelFrame(self.root, text="Geschwindigkeit", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="x", expand=1)

        l = Label(group, text="Parcour-/Strecken-Länge [m]:")
        l.pack(side="left") # fill=BOTH, expand=1)

        self.ev = Entry(group, text = "0", width=10) # , command = self.root.destroy)
        self.ev.pack(padx=10, pady=10, side="left") #fill=BOTH, expand=1)

        self.lv = Label(group, text="Geschwindigkeit: -- m/s, -- km/h")
        self.lv.pack(fill=BOTH, expand=1) # fill=BOTH, expand=1)

        # Empfangsqualitaet
        group = LabelFrame(self.root, text="Empfangsqualität", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="x", expand=1)

        l = Label(group, text="Empfang:")
        l.pack(side="left") # fill=BOTH, expand=1)

        self.qual = Meter(group)
        self.qual.pack(side="left", padx=20) # fill=BOTH) # , expand=1)

        self.lminqual = Label(group, text="Mindest: -- %")
        self.lminqual.pack(side="left", fill=BOTH, expand=1) # fill=BOTH, expand=1)
        self.minqual = 100;

        if root == None:
            b = Button(self.root, text = "Exit", width=10, command = self.root.destroy)
            b.pack() #fill=BOTH, expand=1)

        # State 0: not running, active
        # State 1: running, inactive (2.5s)
        # State 2: running, active
        # State 3: stopped, not released
        self.state = 3
        self.starttime = self.tbest = 0

        self.packet = (200, 0, 0)

        self.pps = libstoppuhr.PPStopUhr()

        # Messprozess starten:
        self.tmeasure = 50 # mS
        self.measure()
        self.updateDisplay()

        if debugging():
            self.tmeasure = 500 # mS
        else:
            tkMessageBox.showwarning("Nicht Vergessen", "Nicht Vergessen: Parcour-/Strecken-Länge eintragen.")

    def reset_best(self):

        if self.tbest != 0 and self.state == 3:
            tkMessageBox.showwarning("Nicht freigegeben", "Keine Freigabe.")

        if self.state == 0:
            self.lbest.config(text="00:00:00")
            self.tbest = 0

    def stop_watch(self):

        if self.state == 2:
            self.packet = (100, 0, self.starttime) # self.pps.timeFunc())
            self.setState3()

    def setState3(self):
        self.state = 3

    def getCourseLen(self):

	    return float(self.ev.get())

    # Geschwindigkeit in m/s errechnen:
    def getSpeed(self, time):
    	return self.getCourseLen() / time

    def freigabe(self):

	    # Pruefung, ob gueltige streckenlaenge eingetragen:
        try:
            self.getCourseLen()
        except ValueError:
            tkMessageBox.showwarning("Eingabefehler", "Parcour-/Strecken-Länge nicht eingetragen.")
            return

        if self.state == 3:
            self.lakt.config(text="00:00:00")
            time.sleep(2.5)
            self.state = 0

    # Debug
    def startstop(self):

        if self.state == 0 or self.state == 2:
            self.pps.startstop()

    def measure(self):

        if self.state == 0 or self.state == 2:

            p = self.pps.recvStartStop()

            if p:

                self.packet = p

                libstoppuhr.sound()

                if self.state == 0:
                    self.starttime = p[2]
                    self.state = 1

                else:
                    self.setState3()

        elif self.state == 1:
            if (self.pps.timeFunc() - self.starttime) > 2.5:
                self.state = 2
        # else: # state == 3
            # if (self.pps.timeFunc() - self.packet[2]) > 2.5:
                # self.state = 0

        #
        # Wir messen zwar nur alle tmeasure mS, das hat aber keinen einfluss auf die
        # genauigkeit der messung, da wir ja den packetcounter haben und 
        # so 'zurückrechnen' können:
        #
        self.root.after(self.tmeasure, self.measure)

    def updateDisplay(self):

        if self.state == 0:
            self.lakt.config(bg="lightgreen")

        elif self.state == 1:

            diff = self.pps.timeFunc() - self.starttime
            self.lakt.config(text="%02d:%02d:%02d" % (diff/60, diff % 60, (diff*100) % 100), bg="red")

            q = (self.packet[0]+1) / 200.0
            self.qual.set(q)
            if q < self.minqual:
                self.lminqual.config(text="Mindest: %d %%" % (q*100))
                self.minqual = q

            self.lv.config(text="Geschwindigkeit: -- m/s, -- km/h")

        elif self.state == 2:

            diff = self.pps.timeFunc() - self.starttime

            self.lakt.config(text="%02d:%02d:%02d" % (diff/60, diff % 60, (diff*100) % 100), bg="lightgreen")


        else: # state = 3

            diff = self.packet[2] - self.starttime
            self.lakt.config(text="%02d:%02d:%02d" % (diff/60, diff % 60, (diff*100) % 100), bg="red")

            v = 0
            try:
                v = self.getSpeed(diff)
            except:
                pass

            if v > 0:
                self.lv.config(text="Geschwindigkeit: %.2f m/s, %.2f km/h" % (v, v*3.6))
            else:
                self.lv.config(text="Geschwindigkeit: -- m/s, -- km/h")


            self.updateBestTime(diff)

            q = (self.packet[0]+1) / 200.0
            self.qual.set(q)
            if q < self.minqual:
                self.lminqual.config(text="Mindest: %d %%" % (q*100))
                self.minqual = q

        self.root.after(int((random.random()+0.5)*50), self.updateDisplay)

    def updateBestTime(self, t):

        if (t > 0) and ((self.tbest == 0) or (t < self.tbest)):
            self.lbest.config(text="%02d:%02d:%02d" % (t/60, t % 60, (t*100) % 100), bg="lightgreen")
            self.tbest = t


if __name__ == "__main__":

    stoppuhr = TkStoppUhr()
    stoppuhr.root.mainloop()






