# coding: utf8
# 
# This file is released under the terms of the GNU General Public License
# as they appear in http://www.gnu.org/licenses/gpl.txt.
#
# Copyright 2009-2017 erwin.rieger@ibrieger.de
#

#
# PMW Demos in: /usr/lib/python2.5/site-packages/Pmw/Pmw_1_2/demos
#
# Nice to have:
#   * Konfiguration fuer ID Startlichtschranke -> Auslösung am ziel wird verhindert

# CVS version: $Revision: 1.15 $ $Date: 2017-05-21 21:18:53 $

from Tkinter import *
import tkMessageBox, string
from tkstop import TkStoppUhr, debugging
import Pmw, pickle, os

# Helper, schreibt unterstrichene überschift in
# datei:
def underlineHeader(f, text, lastchar = "-"):
    f.write(text + "\n")
    f.write(("-" * (len(text) - 1)) + lastchar + "\n")

def header21(f, s):
    f.write("\n\n")
    underlineHeader(f, s)
    f.write("\n")

class StammdatenEingabe(Toplevel):

    def __init__(self, app):

        Toplevel.__init__(self)
        self.title("Starterliste erfassen")

        self.app = app

        group = LabelFrame(self, text="Eingabe Daten Hund", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="x", expand=1)

        self.hncb = Pmw.ComboBox(group,
                    label_text = 'Name:',
                    labelpos = 'nw',
                    # selectioncommand = self.changeName,
                    scrolledlist_items = [],
                    dropdown = 0,
            )
        self.hncb.pack(side = 'left', fill = 'both', expand = 1, padx = 10, pady = 10)

        self.loadNames()

        klassen = []
        for kid in app.klassen:
            klassen.append(app.klassen[kid])

        self.kcb = Pmw.ComboBox(group,
            label_text = 'Klasse:',
            labelpos = 'nw',
            selectioncommand = self.changeKlasse,
            scrolledlist_items = klassen,
            dropdown = 0,
            entryfield_entry_state=DISABLED
            )
        self.kcb.pack(side = 'left', fill = 'both', expand = 1, padx = 10, pady = 10)

        self.snl = Label(group, text="StartNummer: --")
        self.snl.pack(side='left') # fill=BOTH, expand=1)

        # self.sn = Entry(group, text = "-", width=10)
        # self.sn.pack(padx=10, pady=10, side="left") #fill=BOTH, expand=1)

        button = Button(self, text="Speichern", command=self.saveStammdaten)
        button.pack(padx=10, pady=10, side='left')

        button = Button(self, text="Löschen", command=self.deleteStammdaten)
        button.pack(padx=10, pady=10, side='left')

        button = Button(self, text="Schliessen", command=self.destroy)
        button.pack(padx=10, pady=10, side='left')

        self.hncb._entryfield.configure(validate = {'validator': self.validateName})
        self.kcb._entryfield.configure(validate = {'validator': self.validateKlasse})

    def loadNames(self):
        names = []
        for id in app.starterliste:
            names.append(app.starterliste[id][0])
        names.sort()
        self.hncb.component("scrolledlist").setlist(names)
        self.app.loadNames()

    def validateName(self, param):
        
        print "validateName called ...", param
        name = string.strip(self.hncb._entryfield.getvalue())
        self.id = self.app.findName(name)
        print "found id: ", self.id
        if self.id:
            self.kcb.selectitem(
                self.app.klassen[self.app.starterliste[self.id][1]], 1)
            # self.sn.delete(0, END)
            # self.sn.insert(0, `self.app.starterliste[id][2]`)
            self.startnummer = self.app.starterliste[self.id][2]
            self.snl.config(text="StartNummer: " + `self.app.starterliste[self.id][2]`)
        else:
            self.startnummer = -1
            self.snl.config(text="StartNummer: --")

            aktklasse = self.kcb._entryfield.getvalue()
            if aktklasse:
                print "setze klasse: ", aktklasse
                self.validateKlasse(aktklasse)

        return Pmw.OK

    def changeKlasse(self, param):
        print "changeKlasse called ...", param

    def validateKlasse(self, param):

        print "ValidateKlasse called ...", param
        kid = self.app.findKlasse(self.kcb._entryfield.getvalue())
        if not kid:
            return Pmw.OK

        snl = []
        for id in self.app.starterliste:
            if self.app.starterliste[id][1] == kid:
                snl.append(self.app.starterliste[id][2])

        print "startnummern für klasse: ", self.id, kid, snl

        if self.id:
            # aktuelle sn muss in neuer klasse eindeutig sein
            if kid == self.app.starterliste[self.id][1]:
                self.startnummer = self.app.starterliste[self.id][2]
                print "alte sn:", self.startnummer
            else:
                sn = (kid-1)*100
                if sn == 0:
                    sn = 1
                while sn in snl:
                    sn += 1
                    assert(sn < kid*100)
                self.startnummer = sn
                print "neue sn:", self.startnummer
        else:
            # neue startnummer vergeben:
            if snl == []:
                self.startnummer = (kid-1)*100
                if self.startnummer == 0:
                    self.startnummer = 1
            else:
                snl.sort()
                self.startnummer = snl[-1] + 1
            print "neuer starter, neue sn:", self.startnummer

        self.snl.config(text="StartNummer: " + `self.startnummer`)
        return Pmw.OK

    def saveStammdaten(self):

        name = string.strip(self.hncb._entryfield.getvalue())

        if not name:
            tkMessageBox.showwarning("Achtung", "Kein Name angegeben.")
            return

        print "save called ...", name

        kid = self.app.findKlasse(self.kcb._entryfield.getvalue())
        if not kid:
            tkMessageBox.showwarning("Achtung", "Keine Klasse ausgewählt.")
            return

        # sn = 0
        # try:
            # sn = int(self.sn.get())
        # except ValueError:
            # tkMessageBox.showwarning("Achtung", "Ungültige Startnummer.")
            # return

        assert(self.startnummer >= 0)

        id = self.app.findName(name)
        print "found id: ", id
        if id:

            self.dialog = Pmw.MessageDialog(self,
                title = 'Hund existiert bereits',
                message_text = 'Hund existiert bereits',
                iconpos = 'w',
                icon_bitmap = 'warning',
                command = self.dialogCmd,
                buttons = ('Speichern', 'Abbrechen'))
        else:
            # Neuer Eintrag:
            app.starterliste[app.newId()] = (name, kid, self.startnummer)

            self.loadNames()

            app.dumpStammdaten()

    def dialogCmd(self, param):
        print "dialogCmd called ...", param
        self.dialog.destroy()

        if param == "Speichern":
            name = string.strip(self.hncb._entryfield.getvalue())
            kid = self.app.findKlasse(self.kcb._entryfield.getvalue())
            app.starterliste[app.findName(name)] = (name, kid, self.startnummer)

            self.loadNames()

            app.dumpStammdaten()

    def deleteStammdaten(self):

        name = string.strip(self.hncb._entryfield.getvalue())

        if not name:
            tkMessageBox.showwarning("Achtung", "Kein Name angegeben.")
            return

        print "delete called ...", name

        self.id = self.app.findName(name)
        print "found id: ", self.id
        if self.id:

            self.dialog = Pmw.MessageDialog(self,
                title = 'Starter löschen ',
                message_text = 'Starter "'+ name +'" und seine Ergebnisse löschen?',
                iconpos = 'w',
                icon_bitmap = 'warning',
                command = self.deleteCmd,
                buttons = ('Löschen', 'Abbrechen'))

    def deleteCmd(self, param):
        print "deleteCmd called ...", param
        self.dialog.destroy()

        if param == "Löschen":
            
            del app.starterliste[self.id]

            self.loadNames()

            app.dumpStammdaten()

            if self.id in app.results:

                del app.results[self.id]

                app.dumpResults()

    def __del__(self):
        print "StammdatenEingabe is dying ..."

class StarterAuswahl(Toplevel):

    def __init__(self, name, app, uebernahmeDialog):

        Toplevel.__init__(self)
        self.title("Starter auswählen")

        self.app = app
        self.uebernahmeDialog = uebernahmeDialog

        group = LabelFrame(self, text="Starter auswählen", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="x", expand=1)

        names = []
        for id in app.starterliste:

            n = app.starterliste[id][0]
            kid = app.starterliste[id][1]
            klasse = app.klassen[kid]

            names.append("%s (%s)" % (n, klasse))

        names.sort()

        self.hncb = Pmw.ComboBox(group,
                    label_text = 'Name:',
                    labelpos = 'nw',
                    # selectioncommand = self.changeName,
                    scrolledlist_items = names,
                    dropdown = 0,
                    entryfield_entry_state=DISABLED,
            )
        self.hncb.pack(side = 'left', fill = 'both', expand = 1, padx = 10, pady = 10)

        id = app.findName(name)
        kid = app.starterliste[id][1]
        klasse = app.klassen[kid]
        self.hncb.selectitem("%s (%s)" % (name, klasse))

        button = Button(self, text="Übernehmen", command=self.uebernehmen)
        button.pack(padx=10, pady=10, side='left')

        button = Button(self, text="Abbrechen", command=self.cancel)
        button.pack(padx=10, pady=10, side='left')

    def uebernehmen(self):

        aktname = self.hncb._entryfield.getvalue()

        for id in self.app.starterliste:

            name = self.app.starterliste[id][0]
            kid = self.app.starterliste[id][1]
            klasse = self.app.klassen[kid]

            n = "%s (%s)" % (name, klasse)

            if aktname == n:

                id = self.app.findName(name)

                self.app.kcb.selectitem(
                        self.app.klassen[self.app.starterliste[id][1]], 1)

                self.app.loadNames()

                self.app.hncb.selectitem(name, 1)

                self.uebernahmeDialog.destroy()

                self.app.setState3()

                self.destroy()

                break

    def cancel(self):

        self.destroy()

class TWAgi(TkStoppUhr):

    def __init__(self):

        self.root = Tk()

        Pmw.initialise(self.root)

        # self.root.title("Zeitmessung - Ing. Buero Rieger")

        # Menue:
        menubar = Menu(self.root)

        filemenu = Menu(menubar, tearoff=0)
        # filemenu.add_command(label="Open", command=hello)
        # filemenu.add_command(label="Save", command=hello)
        # filemenu.add_separator()
        filemenu.add_command(label="Beenden", command=self.root.quit)
        menubar.add_cascade(label="Datei", menu=filemenu)

        datamenu = Menu(menubar, tearoff=0)
        datamenu.add_command(label="Verwaltung Starter", command=self.stammdaten)
        datamenu.add_command(label="Verwaltung Ergebnisse", command=self.editErgebnisse)
        menubar.add_cascade(label="Daten", menu=datamenu)

        amenu = Menu(menubar, tearoff=0)
        amenu.add_command(label="Starterliste", command=self.startliste)
        amenu.add_command(label="Ergebnisse", command=self.ergebnisse)
        amenu.add_command(label="Bestenliste", command=self.bestenliste)
        menubar.add_cascade(label="Auswertung", menu=amenu)

        self.root.config(menu=menubar)

        # Key: id
        # Value: (namehund, klasse, startnummer)
        self.starterliste = {}
        try:
            input = open('stammdaten.pkl', 'rb')
            self.starterliste = pickle.load(input)
            input.close()
        except IOError:
            pass

        self.klassen = {
            1: "Mini",
            2: "Midi",
            3: "Maxi",
            4: "XXL",
            5: "Welpen",
            6: "TestKlasse",
        }

        # # Zähler zur vergabe der startnummern pro klasse
        # self.startnummern = []
        # sn = 1
        # for k in self.klassen:
            # self.startnummern.append(sn)
            # sn += 200

        # Key: id
        # Value: [ Zeiten... ]
        self.results = {}
        try:
            input = open('results.pkl', 'rb')
            self.results = pickle.load(input)
            input.close()
        except IOError:
            pass

        group = LabelFrame(self.root, text="Starter", padx=5, pady=5)
        group.pack(padx=10, pady=10, fill="both", expand=1)

        klassen = []
        for kid in self.klassen:
            klassen.append(self.klassen[kid])

        self.kcb = Pmw.ComboBox(group,
                    label_text = 'Klasse:',
                    labelpos = 'nw',
                    selectioncommand = self.changeKlasse,
                    scrolledlist_items = klassen,
                    dropdown = 1,
            entryfield_entry_state=DISABLED
            )
        self.kcb.pack(side = 'left', fill = 'both', expand = 1, padx = 10, pady = 10)

        names = []
        self.hncb = Pmw.ComboBox(group,
                    label_text = 'Starter:',
                    labelpos = 'nw',
                    # selectioncommand = self.changeName,
                    scrolledlist_items = names,
                    dropdown = 1,
            entryfield_entry_state=DISABLED
            )
        self.hncb.pack(side = 'left', fill = 'both', expand = 1, padx = 10, pady = 10)

        # self.hncb._entryfield.configure(state = 'disabled')

        TkStoppUhr.__init__(self, self.root)

        # xxx
        # self.stammdaten()

        if debugging():
            self.root.bind("<Control-x>", self.enterDefaultValues)

    def enterDefaultValues(self, event):

        self.ev.delete(0, END)
        self.ev.insert(INSERT, "50")

        self.kcb.selectitem("TestKlasse", 1)

        self.loadNames()

    def loadNames(self):

        kid = self.findKlasse(self.kcb._entryfield.getvalue())
        if kid:

            aktname = self.hncb._entryfield.getvalue()

            names = []
            for id in self.starterliste:
                if self.starterliste[id][1] == kid:
                    names.append(self.starterliste[id][0])
    
            names.sort()
            self.hncb.component("scrolledlist").setlist(names)

            if not names:
                self.hncb.component("entryfield").setvalue("")
                return

            if aktname not in names:
                # Starter wurde gelöscht
                self.hncb.selectitem(0, 1)


    def changeKlasse(self, param):
        print "changeKlasse called ...", param

        self.loadNames()

        
    # Parent Methode überschreiben:
    def freigabe(self):

        if self.state == 3:
            if self.hncb.component("entryfield").getvalue():
                TkStoppUhr.freigabe(self)
            else:
                tkMessageBox.showwarning("Achtung", "Kein Starter ausgewählt.")

    # Parent Methode überschreiben:
    def setState3(self):
        TkStoppUhr.setState3(self)
        diff = self.packet[2] - self.starttime

        if diff <= 0:
            # Manueller stop ...
            return

        n = self.hncb.component("entryfield").getvalue()
        s = "Gemessene Zeit %.2f für Starter %s übernehmen?" % (diff, n)
        self.dialog = Pmw.MessageDialog(self.root,
            title = 'Zeit übernehmen',
            message_text = s,
            iconpos = 'w',
            icon_bitmap = 'warning',
            command = self.dialogCmd,
            buttons = ('Übernehmen', 'Starter ändern', 'Verwerfen'),
            defaultbutton = 'Übernehmen')

    def dialogCmd(self, param):
        print "dialogCmd called ...", param

        n = self.hncb.component("entryfield").getvalue()

        if param == "Übernehmen":

            id = self.findName(n)

            diff = self.packet[2] - self.starttime

            speed = self.getSpeed(diff)

            if id in self.results:
                self.results[id].append({"time": diff, "speed": speed})
            else:
                self.results[id] = [{"time": diff, "speed": speed}]

            self.dumpResults()

            self.dialog.destroy()

        elif param == "Verwerfen":

            self.dialog.destroy()

        else:

            StarterAuswahl(n, self, self.dialog)

    def stammdaten(self):
        StammdatenEingabe(self)

    def startliste(self):
        f = open("/tmp/startliste.txt", "w")

        s = "STARTERLISTE (%d Starter):" % len(self.starterliste)
        header21(f, s)

        for kid in self.klassen:
            l = []
            for id in self.starterliste:
                if kid == self.starterliste[id][1]:
                    l.append((id, self.starterliste[id]))

            if l != []:
                s = "Klasse " + self.klassen[kid] + ":"
                header21(f, s)

                s = "    %10s | %15s | %30s |" % ("ID", "Startplatz", "Name")
                underlineHeader(f, s, "+")

                for tup in l:
                    f.write("    %10d | %15d | %30s |\n" % (tup[0], tup[1][2], tup[1][0]))

        f.close()

        os.system("mousepad /tmp/startliste.txt&")

    def editErgebnisse(self):
        pass

    def ergebnisse(self):
        f = open("/tmp/Ergebnisse.txt", "w")

        nstarts = 0
        for id in self.results:
            nstarts = nstarts + len(self.results[id])

        s = "ERGEBNISSE (%d Starts):" % nstarts
        header21(f, s)

        for kid in self.klassen:
            l = []
            for id in self.starterliste:
                if id in self.results and kid == self.starterliste[id][1]:
                    l.append((id, self.starterliste[id], self.results[id]))

            if l != []:

                s = "Klasse " + self.klassen[kid] + ":"
                header21(f, s)

                s = "%4s |%11s |%30s |%9s |%13s |%14s |" % ("ID", "Startplatz", "Name", "Zeit[s]", "Geschw.[m/s]", "Geschw.[km/s]")
                underlineHeader(f, s)

                for tup in l:
                    for z in tup[2]:
                        f.write("%4d |%11d |%30s |%9.2f |%13.2f |%14.2f |\n" % (tup[0], tup[1][2], tup[1][0], z['time'], z['speed'], z['speed']*3.6))

        f.close()

        os.system("mousepad /tmp/Ergebnisse.txt&")


    def bestenliste(self):
        
        # fuer jeden starter der klasse das beste ergebnis suchen:
        f = open("/tmp/Rangliste.txt", "w")

        s = "RANGLISTE (%d Starter):" % len(self.starterliste)
        header21(f, s)

        for kid in self.klassen:
            bestd = {}

            for id in self.starterliste:
                if id in self.results and kid == self.starterliste[id][1]:
                    rl = self.results[id]
                    print "rl: ", rl
                    rl.sort(key = lambda result: result["time"])
                    best = rl[0]['time']

                    bestd[best] = (id, rl[0]['speed'])

            if bestd != {}:

                s = "Klasse " + self.klassen[kid] + ":"
                header21(f, s)

                s = "%4s |%11s |%30s |%9s |%13s |%14s |" % ("Rang", "Startplatz", "Name", "Zeit[s]", "Geschw.[m/s]", "Geschw.[km/s]")
                underlineHeader(f, s)

                zl = bestd.keys()
                zl.sort()

                i = 1
                for best in zl:
                    id = bestd[best][0]
                    f.write("%4d |%11d |%30s |%9.2f |%13.2f |%14.2f |\n" % 
                        (i, self.starterliste[id][2], self.starterliste[id][0], best, bestd[best][1], bestd[best][1]*3.6))
                    i += 1

        f.close()

        os.system("mousepad /tmp/Rangliste.txt&")


    def findName(self, name):
        for id in self.starterliste:
            if self.starterliste[id][0] == name:
                return id
        return None

    def findKlasse(self, name):
        for id in self.klassen:
            if self.klassen[id] == name:
                return id
        return None

    def newId(self):
        vl = self.starterliste.keys()
        if vl == []:
            return 1
        vl.sort()
        return vl[-1] + 1

    def dumpStammdaten(self):

        print "saving to file ..."
        os.system("mv stammdaten.pkl stammdaten.bak")
        output = open('stammdaten.pkl', 'wb')
        pickle.dump(app.starterliste, output)
        output.close()

    def dumpResults(self):

        print "saving to file ..."
        os.system("mv results.pkl results.bak")
        output = open('results.pkl', 'wb')
        pickle.dump(self.results, output)
        output.close()

    """
    def bestZeit(self):
        
        # Bestzeit fuer akt. klasse anzeigen
        kid = self.findKlasse(self.kcb._entryfield.getvalue())

        f = open("/tmp/Rangliste.txt", "w")

        for kid in self.klassen:
            bestd = {}

            for id in self.starterliste:
                if id in self.results and kid == self.starterliste[id][1]:
                    rl = self.results[id]
                    rl.sort()
                    best = rl[0]

                    bestd[best] = id

            if bestd != {}:
                f.write("\nKlasse " + self.klassen[kid] + ":\n\n")
                f.write("%5s|%12s|%30s|%10s|\n" % ("Rang", "Startplatz", "Name", "Zeit [s]"))

                zl = bestd.keys()
                zl.sort()

                i = 1
                for best in zl:
                    id = bestd[best]
                    f.write("%5d|%12d|%30s|%10.2f|\n" % (i, self.starterliste[id][2], 
                        self.starterliste[id][0], best))
                    i += 1

        f.close()

        os.system("mousepad /tmp/Rangliste.txt&")
    """

    # Parent Methode überschreiben:
    def updateBestTime(self, t):

        if (self.tbest == 0) or (t < self.tbest):
            self.lbest.config(text="%02d:%02d:%02d" % (t/60, t % 60, (t*100) % 100), bg="lightgreen")
            self.tbest = t



app = TWAgi()
app.root.mainloop()


